#ifndef ODE_SOLVER_1_1
#define ODE_SOLVER_1_1
/************************************************
 * ODE solver header file                       *
 *                                              *
 * (C) Alessandro Colombo, last update 11/12/09 *
 *                                              *
 ************************************************/

/* uncomment the next line, and comment the following one, to activate debug messages in the code*/
/*#define DEBUG*/
#undef DEBUG


#include "dynamical_systems_types.h"

int ode_solver(const SYS_DEF *sys_def, const INT_TOL *int_tol, double **t_out, double ***state_out, unsigned long int *n_el_out, const OPTIONS *options);
/* the output vectors t_out and state_out are allocated by the function, but must be deallocated by the user.  n_el_out is the number of elements in the output vectors */

int ode_solver_noplot(const SYS_DEF *sys_def, const INT_TOL *int_tol, double *t_out, double state_out[], const OPTIONS *options); 
/* this returns the last element only.  The output vector must be allocated before calling the function */

int ode_solver_events(const SYS_DEF *sys_def, const INT_TOL *int_tol, const EVENT_DEF *event_def, const EVENT_TOL *event_tol, double **t_out, double ***state_out, unsigned long int *n_el_out, int *event_detected, int *convergence, const OPTIONS *options);
/* This is an integrator with event detection.  Integration is stopped when an event is detected.   The output vectors t_out and state_out are allocated by the function, but must be deallocated by the user.  Event_detected is -1 if no event was detected, otherwise it returns the index of the detected function, (0..n-1 for n event functions).  Convergence is 0 if the event detection algorithm didn't converge, 1 if it converged.  eventfun is a pointer to an event function eventfun(const double x[], const double t, void *pars).  Notice that, if an event is detected, the endpoint lies on the event surface or before it, so that restarting the integration with no further changes will result in the same event being detected again.
*/
		
int ode_solver_events_noplot(const SYS_DEF *sys_def, const INT_TOL *int_tol, const EVENT_DEF *event_def, const EVENT_TOL *event_tol, double *t_out, double state_out[], int *event_detected, int *convergence, const OPTIONS *options); 
/* this returns the last element only, with event detection.  The return value corresponds to the event coordinates and time, or to the end point and time if no event is detected.  The output vectors must be allocated before calling the function.
Event_detected is -1 if no event was detected, otherwise it returns the index of the detected function, (0..n-1 for n event functions).  Convergence is 0 if the event detection algorithm didn't converge, 1 if it converged.  eventfun is a pointer to an event function eventfun(const double x[], const double t, void *pars). Notice that, if an event is detected, the endpoint lies on the event surface or before it, so that restarting the integration with no further changes will result in the same event being detected again.
*/
#endif