#ifndef SMALL_FUNCTIONS_1_1
#define SMALL_FUNCTIONS_1_1
/************************************************
 * Functions library							*
 *                                              *
 * (C) Alessandro Colombo, last update 15/12/10 *
 *												*
 ************************************************/

#include "dynamical_systems_types.h"

/*types definition*/
typedef struct{
	double time;
	void *pars;
	int sysdim;
	const gsl_odeiv_step_type* method;
	EVENT_FUNC_P ev_function;
	RHS_FUNC_P sys_function;
	JAC_FUNC_P jac_function;
	double *state;
	int event_index;
}EVENT_WRAPPER_PARS;

/*wrapper for EVENTFUN, to prepare it to be used in the root-finding algorithm*/
inline double event_wrapper(double t, void * ewpars);

/*other auxiliary functions */
inline int integration_step(const SYS_DEF *sys_def, const INT_TOL *int_tol, gsl_odeiv_system sys, 
							gsl_odeiv_evolve *evol, gsl_odeiv_control *ctrl, gsl_odeiv_step *step, 
							double *step_size, double *state, double * old_state,
							double * current_time, double * old_time, double next_out_time);

inline int positive_variables_check(const SYS_DEF *sys_def, const OPTIONS *options, const gsl_odeiv_system sys,
									gsl_odeiv_evolve *evol, gsl_odeiv_step *step, double * step_size, 
									double *state, double * old_state, double * current_time, const double * old_time);

inline int setup_integrator_and_initcond(const SYS_DEF *sys_def, const INT_TOL *int_tol, gsl_odeiv_system *sys, 
										 gsl_odeiv_evolve **evol, gsl_odeiv_control **ctrl, gsl_odeiv_step **step,
										 const gsl_odeiv_step_type **step_type, double * step_size,
										 double *state, double * current_time);

inline int output_vectors_allocation(const SYS_DEF *sys_def, const INT_TOL *int_tol, unsigned long int * size_vec, 
									 unsigned long int *n_el_out, double *** state_out, double *state, double **t_out);

inline int setup_event(const SYS_DEF *sys_def, EVENT_WRAPPER_PARS *ewpars, const EVENT_DEF *event_def, int * last_sign, 
					   int *event_detected, int *convergence);

inline int event_check(const SYS_DEF * sys_def, EVENT_WRAPPER_PARS * ewpars, const EVENT_DEF *event_def, 
					   const EVENT_TOL *event_tol,  gsl_odeiv_system *sys, gsl_odeiv_step *step, int * last_sign,
					   double * state, double * current_time, int * event_detected, int * convergence);

inline int resize_output(const SYS_DEF * sys_def, double * current_time, unsigned long int * size_vec,
						 unsigned long int *n_el_out, double *** state_out, double **t_out);
						 
inline int shrink_output(const SYS_DEF * sys_def, unsigned long int n_el_out, double *** state_out, double ** t_out);

inline void gsl_vector_normalize(gsl_vector *a, double *norm);

inline void gsl_matrix_gram_schmidt(gsl_matrix *A, gsl_vector *v);
						 
#endif