#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "lyap_exp_1.1.h"

int rhsfun (double time, const double state[], double rhs[], void *pars);

int rhsjac (double time, const double state[], double *drhs_dstate,
	    double drhs_dtime[], void *pars);

inline int rhs_lin_part(double time, const double state[], double rhs[], void *pars);

int
main (void)
{
/* LINEAR SYSTEM PARAMETERS */
/*  double pars[3] = {-1e-2,-5e-2,-1e-3};*/
  
/* ROESSLER PARAMETERS */
/* double pars[3] = {0.2,0.2,5.7};  */

/* LORENTZ PARAMETERS (p,r,b)*/
  double pars[3]={10.0,28.0,8.0/3.0};
 
  SYS_DEF sysdef;
  INT_TOL inttol;
  LYAP_TOL lyaptol;
  OPTIONS options;
  int i, flag;
  int N_EXP = 3;
  double lexp[N_EXP], state_init[3] = { 1.0, 2.0, 1.0 };	//this is just a random initial vector
  double tout;
  double stateout[3];



  clock_t cl_init, cl_final;

  sysdef.rhsfun = rhsfun;
  sysdef.jacfun=NULL;
  //sysdef.jacfun = rhsjac;
  //sysdef.rhs_lin_part = NULL;
  sysdef.rhs_lin_part =  rhs_lin_part;
  sysdef.t0 = 0.0;
  sysdef.tf = 1e5;
  sysdef.x0 = state_init;
  sysdef.pars = pars;
  sysdef.dt = 0.0;
  sysdef.sysdim = 3;
  sysdef.int_method = gsl_odeiv_step_rkf45;
  //sysdef.int_method = gsl_odeiv_step_rk2;

  inttol.rel_tol = 1.0E-10;
  inttol.abs_tol = 1.0E-10;
  inttol.max_step_size = 1.0E0;
  inttol.min_step_size=1E-4;

  lyaptol.lyap_tol = 1e-3;
  lyaptol.min_time = 1000;
  lyaptol.test_interval = 100;
  lyaptol.max_ratio=1e2;
  lyaptol.min_ratio=1e-2;
  lyaptol.rescaling_factor=1000.0;

  options.n_positive = 0;
  options.lyapexp_method = QR;

  cl_init = clock ();

  for (i = 0; i < 3; i++)
    printf ("parameter %d = %f\n", i, pars[i]);

  flag =
    lyap_exp (&sysdef, &inttol, &lyaptol, N_EXP, lexp, &tout,
		       stateout, &options);
  printf ("Lyapunov exponents = [");
  for (i = 0; i < N_EXP; i++)
    printf (" %E", lexp[i]);
  printf (" ]\n");
  printf ("state_init=[");
  for (i = 0; i < 3; i++)
    printf (" %E", state_init[i]);
  printf (" ]\n");
  printf ("output flag = %d\n", flag);
  printf ("final time = %f\n", tout);
  printf ("final state = [%f, %f, %f]\n", stateout[0],stateout[1],stateout[2]);
  fflush (stdout);

  cl_final = clock ();
  printf ("elapsed time: %f seconds\n",
	  (double) (cl_final - cl_init) / CLOCKS_PER_SEC);

  return 0;
}

/* LINEAR SYSTEM */
/*
int
rhsfun (double time, const double state[], double rhs[], void *pars)
{
  double a = ((double *) pars)[0];
  double b = ((double *) pars)[1];
  double c = ((double *) pars)[2];
  
  rhs[0] = a*state[0];
  rhs[1] = b*state[1];
  rhs[2] = c*state[2];

  return 0;
}

int
rhsjac (double time, const double state[], double *drhs_dstate,
	double drhs_dtime[], void *pars)
{
  double a = ((double *) pars)[0];
  double b = ((double *) pars)[1];
  double c = ((double *) pars)[2];

  drhs_dstate[0] = a;
  drhs_dstate[1] = 0;
  drhs_dstate[2] = 0;
  drhs_dstate[3] = 0;
  drhs_dstate[4] = b;
  drhs_dstate[5] = 0;
  drhs_dstate[6] = 0;
  drhs_dstate[7] = 0;
  drhs_dstate[8] = c;

  return 0;
}
int rhs_lin_part(double time, const double state[], double rhs[], void *pars)
{
  double a = ((double *) pars)[0];
  double b = ((double *) pars)[1];
  double c = ((double *) pars)[2];
  
  rhs[0] = a*state[0];
  rhs[1] = b*state[1];
  rhs[2] = c*state[2];
  
  rhs[3] = a*state[3];
  rhs[4] = b*state[4];
  rhs[5] = c*state[5];
  
  return 0;
}
*/

/* ROESSLER SYSTEM : */
/*
int
rhsfun (double time, const double state[], double rhs[], void *pars)
{
  double a = ((double *) pars)[0];
  double b = ((double *) pars)[1];
  double c = ((double *) pars)[2];

  rhs[0] = (-state[1] - state[2]);
  rhs[1] = (state[0] + a * state[1]);
  rhs[2] = (b + state[0] * state[2] - c * state[2]);

  return 0;
}

int
rhsjac (double time, const double state[], double *drhs_dstate,
	double drhs_dtime[], void *pars)
{
  double a = ((double *) pars)[0];
  double b = ((double *) pars)[1];
  double c = ((double *) pars)[2];

  drhs_dstate[0] = 0;
  drhs_dstate[1] = -1;
  drhs_dstate[2] = -1;
  drhs_dstate[3] = 1;
  drhs_dstate[4] = a;
  drhs_dstate[5] = 0;
  drhs_dstate[6] = state[2];
  drhs_dstate[7] = 0;
  drhs_dstate[8] = (state[0] - c);

  return 0;
}

int rhs_lin_part(double time, const double state[], double rhs[], void *pars){
  double a = ((double *) pars)[0];
  double b = ((double *) pars)[1];
  double c = ((double *) pars)[2];

  rhs[0] = (-state[1] - state[2]);
  rhs[1] = (state[0] + a * state[1]);
  rhs[2] = (b + state[0] * state[2] - c * state[2]);
  
  rhs[3] = -state[4]-state[5];
  rhs[4] = state[3]+a*state[4];
  rhs[5] = state[2]*state[3]+(state[0] - c)*state[5];
  return 0;
}
*/

/*LORENZ SYSTEM*/

int rhsfun(double time, const double state[], double rhs[], void *pars){
	double p=((double*)pars)[0];
	double r=((double*)pars)[1];
	double b=((double*)pars)[2];
	
	rhs[0]=p*(state[1]-state[0]);
	rhs[1]=-state[0]*state[2]+r*state[0]-state[1];
	rhs[2]=state[0]*state[1]-b*state[2];

	return 0;
}

int rhsjac(double time, const double state[], double *drhs_dstate, double drhs_dtime[], void *pars){
	double p=((double*)pars)[0];
	double r=((double*)pars)[1];
	double b=((double*)pars)[2];
	
	drhs_dstate[0]=-p;
	drhs_dstate[1]=p;
	drhs_dstate[2]=0.0;
	drhs_dstate[3]=-state[2]+r;
	drhs_dstate[4]=-1.0;
	drhs_dstate[5]=-state[0];
	drhs_dstate[6]=state[1];
	drhs_dstate[7]=state[0];
	drhs_dstate[8]=-b;

	return 0;
}

int rhs_lin_part(double time, const double state[], double rhs[], void *pars){
  double p=((double*)pars)[0];
  double r=((double*)pars)[1];
  double b=((double*)pars)[2];
	
  rhs[0]=p*(state[1]-state[0]);
  rhs[1]=-state[0]*state[2]+r*state[0]-state[1];
  rhs[2]=state[0]*state[1]-b*state[2];
  
  rhs[3]=-p*state[3]+p*state[4];
  rhs[4]=(-state[2]+r)*state[3]-state[4]-state[0]*state[5];
  rhs[5]=state[1]*state[3]+state[0]*state[4]-b*state[5];
  
  return 0;
}
